@@ -1,6 +1,7 @@
 {-# LANGUAGE RankNTypes #-}
 {-# OPTIONS -Wall #-}
 
+
 -- |
 -- Module      : Text.Format
 -- Copyright   : (c) 2013 Chris Done, 2013 Shachaf Ben-Kiki
@@ -14,6 +15,7 @@
 -- Example:
 --
 -- >>> format ("Person's name is " % text % ", age is " % hex) "Dave" 54
+-- "Person's name is Dave, age is 36"
 --
 -- See "Formatting.Formatters" for a list of formatters.
 -- See "Formatting.Combinators" for a list of formatting combinators, for combining and altering formatters.
@@ -22,7 +24,9 @@ module Formatting
   (
   Format,
   (%),
+  (%+),
   (%.),
+  (<%+>),
   now,
   later,
   mapf,
@@ -37,11 +41,17 @@ module Formatting
   hprint,
   hprintLn,
   formatToString,
+  formatted,
   -- * Formatting library
   module Formatting.Formatters,
   module Formatting.Combinators
  ) where
 
+
 import Formatting.Formatters
 import Formatting.Combinators
+import Formatting.FromBuilder
 import Formatting.Internal
+
+-- $setup
+-- >>> :set -XOverloadedStrings
