@@ -4,11 +4,11 @@ module Formatting.Internal.Raw
   (
   module Data.Text.Format,
   module Data.Text.Format.Functions,
-  module Data.Text.Format.Int,
+  module Data.Text.Lazy.Builder.Int,
   module Data.Text.Format.Types
  ) where
 
 import Data.Text.Format
 import Data.Text.Format.Functions
-import Data.Text.Format.Int
+import Data.Text.Lazy.Builder.Int
 import Data.Text.Format.Types
